@@ -1,4 +1,4 @@
-/* $OpenBSD: msg.c,v 1.16 2015/01/15 09:40:00 djm Exp $ */
+/* $OpenBSD: msg.c,v 1.20 2020/10/18 11:32:01 djm Exp $ */
 /*
  * Copyright (c) 2002 Markus Friedl.  All rights reserved.
  *
@@ -45,16 +45,16 @@ ssh_msg_send(int fd, u_char type, struct sshbuf *m)
 	u_char buf[5];
 	u_int mlen = sshbuf_len(m);
 
-	debug3("ssh_msg_send: type %u", (unsigned int)type & 0xff);
+	debug3_f("type %u", (unsigned int)type & 0xff);
 
 	put_u32(buf, mlen + 1);
 	buf[4] = type;		/* 1st byte of payload is mesg-type */
 	if (atomicio(vwrite, fd, buf, sizeof(buf)) != sizeof(buf)) {
-		error("ssh_msg_send: write");
+		error_f("write: %s", strerror(errno));
 		return (-1);
 	}
-	if (atomicio(vwrite, fd, (u_char *)sshbuf_ptr(m), mlen) != mlen) {
-		error("ssh_msg_send: write");
+	if (atomicio(vwrite, fd, sshbuf_mutable_ptr(m), mlen) != mlen) {
+		error_f("write: %s", strerror(errno));
 		return (-1);
 	}
 	return (0);
@@ -71,21 +71,21 @@ ssh_msg_recv(int fd, struct sshbuf *m)
 
 	if (atomicio(read, fd, buf, sizeof(buf)) != sizeof(buf)) {
 		if (errno != EPIPE)
-			error("ssh_msg_recv: read: header");
+			error_f("read header: %s", strerror(errno));
 		return (-1);
 	}
 	msg_len = get_u32(buf);
-	if (msg_len > 256 * 1024) {
-		error("ssh_msg_recv: read: bad msg_len %u", msg_len);
+	if (msg_len > sshbuf_max_size(m)) {
+		error_f("read: bad msg_len %u", msg_len);
 		return (-1);
 	}
 	sshbuf_reset(m);
 	if ((r = sshbuf_reserve(m, msg_len, &p)) != 0) {
-		error("%s: buffer error: %s", __func__, ssh_err(r));
+		error_fr(r, "reserve");
 		return -1;
 	}
 	if (atomicio(read, fd, p, msg_len) != msg_len) {
-		error("ssh_msg_recv: read: %s", strerror(errno));
+		error_f("read: %s", strerror(errno));
 		return (-1);
 	}
 	return (0);
