@@ -1,4 +1,4 @@
-/* 	$OpenBSD: tests.c,v 1.6 2021/03/19 04:23:50 djm Exp $ */
+/* 	$OpenBSD: tests.c,v 1.7 2021/05/21 03:48:07 djm Exp $ */
 /*
  * Regress test for misc helper functions.
  *
@@ -10,7 +10,9 @@
 #include <sys/types.h>
 #include <sys/param.h>
 #include <stdio.h>
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#endif
 #include <stdlib.h>
 #include <string.h>
 
@@ -23,6 +25,7 @@ void test_parse(void);
 void test_convtime(void);
 void test_expand(void);
 void test_argv(void);
+void test_strdelim(void);
 
 void
 tests(void)
@@ -31,4 +34,5 @@ tests(void)
 	test_convtime();
 	test_expand();
 	test_argv();
+	test_strdelim();
 }
